@@ -25,8 +25,10 @@
 #include <X11/X.h>
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
-#ifdef HAVE_LIBXINERAMA
+#ifdef HAVE_XFREE_XINERAMA
 #include <X11/extensions/Xinerama.h>
+#elif HAVE_SOLARIS_XINERAMA
+#include <X11/extensions/xinerama.h>
 #endif
 #include <pwd.h>
 #include <unistd.h>
@@ -44,6 +46,11 @@ struct _GdmWindow {
 	gboolean center; /* do centering */
 	gboolean recenter; /* do re-centering */
         gboolean takefocus; /* permit take focus */
+
+	/* hack, when we reparent, we will get an unmap and then
+	 * an map, and we want to ignore those */
+	int ignore_next_map;
+	int ignore_next_unmap;
 };
 
 static GList *windows = NULL;
@@ -86,7 +93,7 @@ gdm_wm_screen_init (int cur_screen_num)
 	}
 
 	{
-#ifdef HAVE_LIBXINERAMA
+#ifdef HAVE_XFREE_XINERAMA
 	gboolean have_xinerama = FALSE;
 
 	gdk_flush ();
@@ -135,6 +142,67 @@ gdm_wm_screen_init (int cur_screen_num)
 
 		XFree (xscreens);
 	} else
+#elif HAVE_SOLARIS_XINERAMA
+	gboolean have_xinerama = FALSE;
+ 	/* This code from GDK, Copyright (C) 2002 Sun Microsystems */
+ 	int opcode;
+	int firstevent;
+	int firsterror;
+	int n_monitors = 0;
+	
+	gdk_flush ();
+	gdk_error_trap_push ();
+	have_xinerama = XQueryExtension (GDK_DISPLAY (),
+			"XINERAMA",
+			&opcode,
+			&firstevent,
+			&firsterror);
+	gdk_flush ();
+	if (gdk_error_trap_pop () != 0)
+		have_xinerama = FALSE;
+
+	if (have_xinerama) {
+		int i;
+		int result;
+		XRectangle monitors[MAXFRAMEBUFFERS];
+		unsigned char  hints[16];
+		
+		result = XineramaGetInfo (GDK_DISPLAY (), 0, monitors, hints, &n_monitors);
+		/* Yes I know it should be Success but the current implementation 
+		 * returns the num of monitor
+		 */
+
+		if (result <= 0) {
+			/* should NEVER EVER happen */
+			syslog (LOG_ERR, "Xinerama active, but <= 0 screens?");
+			gdm_wm_screen.x = 0;
+			gdm_wm_screen.y = 0;
+			gdm_wm_screen.width = gdk_screen_width ();
+			gdm_wm_screen.height = gdk_screen_height ();
+
+			gdm_wm_allscreens = g_new0 (GdkRectangle, 1);
+			gdm_wm_allscreens[0] = gdm_wm_screen;
+			gdm_wm_screens = 1;
+			return;
+		}
+
+		if (n_monitors <= cur_screen_num)
+			cur_screen_num = 0;
+
+		gdm_wm_allscreens = g_new0 (GdkRectangle, n_monitors);
+		gdm_wm_screens = n_monitors;
+
+		for (i = 0; i < n_monitors; i++) {
+			gdm_wm_allscreens[i].x = monitors[i].x;
+			gdm_wm_allscreens[i].y = monitors[i].y;
+			gdm_wm_allscreens[i].width = monitors[i].width;
+			gdm_wm_allscreens[i].height = monitors[i].height;
+
+			if (cur_screen_num == i)
+				gdm_wm_screen = gdm_wm_allscreens[i];
+		}
+
+	} else
 #endif
 	{
 		gdm_wm_screen.x = 0;
@@ -162,13 +230,13 @@ gdm_wm_set_screen (int cur_screen_num)
 void
 gdm_wm_center_window (GtkWindow *cw) 
 {
-	GtkRequisition req;
         gint x, y;
+        gint w, h;
 
-	gtk_widget_size_request (GTK_WIDGET (cw), &req);
+	gtk_window_get_size (cw, &w, &h); 
 
-	x = gdm_wm_screen.x + (gdm_wm_screen.width - req.width)/2;
-	y = gdm_wm_screen.y + (gdm_wm_screen.height - req.height)/2;	
+	x = gdm_wm_screen.x + (gdm_wm_screen.width - w)/2;
+	y = gdm_wm_screen.y + (gdm_wm_screen.height - h)/2;	
 
 	if (x < gdm_wm_screen.x)
 		x = gdm_wm_screen.x;
@@ -187,7 +255,7 @@ trap_push (void)
 	gdk_error_trap_push ();
 }
 
-static gboolean
+static int
 trap_pop (void)
 {
 	trap_depth --;
@@ -569,7 +637,7 @@ has_deco (Window win)
 
 
 static void
-add_deco (GdmWindow *w)
+add_deco (GdmWindow *w, gboolean is_mapped)
 {
 	int x, y;
 	Window root;
@@ -619,9 +687,20 @@ add_deco (GdmWindow *w)
 
 	XMapWindow (wm_disp, w->deco);
 
-	XReparentWindow (wm_disp, w->win, w->deco, 1, 1);
-
+	XSync (wm_disp, False);
 	trap_pop ();
+
+	trap_push ();
+	XReparentWindow (wm_disp, w->win, w->deco, 1, 1);
+	XSync (wm_disp, False);
+	if (trap_pop () == 0) {
+		if (is_mapped) {
+			/* Ignore the next unmap/map, but only
+			 * if reparent window really succeeded */
+			w->ignore_next_map++;
+			w->ignore_next_unmap++;
+		}
+	}
 }
 
 static gboolean
@@ -641,7 +720,7 @@ is_wm_class (XClassHint *hint, const char *string, int len)
 }
 
 static GdmWindow *
-add_window (Window w, gboolean center)
+add_window (Window w, gboolean center, gboolean is_mapped)
 {
 	GdmWindow *gw;
 
@@ -665,6 +744,9 @@ add_window (Window w, gboolean center)
 		gw->recenter = FALSE;
 		gw->takefocus = TRUE;
 
+		gw->ignore_next_map = 0;
+		gw->ignore_next_unmap = 0;
+
 		wmhints = XGetWMHints (wm_disp, w);
 		if (wmhints != NULL) {
 			/* NoInput windows */
@@ -712,7 +794,7 @@ add_window (Window w, gboolean center)
 		gw->y = x;
 
 		center_x_window (gw, w, w);
-		add_deco (gw);
+		add_deco (gw, is_mapped);
 
 		XAddToSaveSet (wm_disp, w);
 
@@ -795,7 +877,9 @@ add_all_current_windows (void)
 
 			if ( ! attribs.override_redirect &&
 			    attribs.map_state != IsUnmapped) {
-				add_window (children[i], FALSE /*center*/);
+				add_window (children[i],
+					    FALSE /*center*/,
+					    TRUE /* is_mapped */);
 			}
 		}
 
@@ -861,7 +945,9 @@ event_process (XEvent *ev)
 		if (gw == NULL) {
 			if (ev->xmaprequest.parent == wm_root) {
 				XGrabServer (wm_disp);
-				gw = add_window (w, TRUE /*center*/);
+				gw = add_window (w,
+						 TRUE /* center */,
+						 FALSE /* is_mapped */);
 				XUngrabServer (wm_disp);
 			}
 		}
@@ -939,17 +1025,26 @@ event_process (XEvent *ev)
 		break;
 	case MapNotify:
 		w = ev->xmap.window;
-		if ( ! ev->xmap.override_redirect &&
-		    focus_new_windows) {
-			gw = find_window (w, FALSE);
-			if (gw != NULL)
+		gw = find_window (w, FALSE);
+		if (gw != NULL) {
+			if (gw->ignore_next_map > 0) {
+				gw->ignore_next_map --;
+				break;
+			}
+			if ( ! ev->xmap.override_redirect &&
+			     focus_new_windows) {
 				gdm_wm_focus_window (w);
+			}
 		}
 		break;
 	case UnmapNotify:
 		w = ev->xunmap.window;
 		gw = find_window (w, FALSE);
 		if (gw != NULL) {
+			if (gw->ignore_next_unmap > 0) {
+				gw->ignore_next_unmap --;
+				break;
+			}
 			XGrabServer (wm_disp);
 			if (gw->deco != None)
 				XUnmapWindow (wm_disp, gw->deco);
@@ -1087,6 +1182,18 @@ gdm_wm_init (Window login_window)
 
 	wm_root = DefaultRootWindow (wm_disp);
 
+	/* set event mask for events on root window */
+	XGetWindowAttributes (wm_disp, wm_root, &attribs);
+	XSelectInput (wm_disp, wm_root,
+		      attribs.your_event_mask |
+		      SubstructureNotifyMask |
+		      SubstructureRedirectMask);
+
+	if (trap_pop () != 0)
+		return;
+
+	trap_push ();
+
 	add_all_current_windows ();
 
 	source = g_source_new (&event_funcs, sizeof (GSource));
@@ -1099,13 +1206,6 @@ gdm_wm_init (Window login_window)
 	g_source_set_can_recurse (source, FALSE);
 	g_source_attach (source, NULL);
 
-	/* set event mask for events on root window */
-	XGetWindowAttributes (wm_disp, wm_root, &attribs);
-	XSelectInput (wm_disp, wm_root,
-		      attribs.your_event_mask |
-		      SubstructureNotifyMask |
-		      SubstructureRedirectMask);
-
 	trap_pop ();
 }
 
@@ -1187,6 +1287,7 @@ gdm_wm_move_window_now (Window window, int x, int y)
 
 		XSync (wm_disp, False);
 		trap_pop ();
+		return;
 	}
 
 	if (gw->deco != None)
@@ -1280,6 +1381,11 @@ gdm_wm_restore_wm_order (void)
 	int i;
 	Atom atom;
 
+	gdk_flush ();
+	XSync (wm_disp, False);
+
+	process_events ();
+
 	gdk_flush ();
 	XSync (wm_disp, False);
 	trap_push ();
