@@ -316,6 +316,7 @@ greeter_ctrl_handler (GIOChannel *source,
 				 GTK_DIALOG_MODAL /* flags */,
 				 GTK_MESSAGE_ERROR,
 				 GTK_BUTTONS_OK,
+				 FALSE /* markup */,
 				 tmp,
 				 /* avoid warning */ "%s", "");
 	g_free (tmp);
@@ -411,6 +412,7 @@ greeter_ctrl_handler (GIOChannel *source,
 					 GTK_DIALOG_MODAL /* flags */,
 					 GTK_MESSAGE_INFO,
 					 GTK_BUTTONS_OK,
+					 FALSE /* markup */,
 					 /* translators:  This is a nice and evil eggie text, translate
 					  * to your favourite currency */
 					 _("Please insert 25 cents "
@@ -539,6 +541,7 @@ greeter_ctrl_handler (GIOChannel *source,
 	break;
 	
     default:
+	gdm_login_abort ("Unexpected greeter command received: '%c'", buf[0]);
 	break;
     }
 
@@ -614,6 +617,7 @@ verify_gdm_version (void)
 				  GTK_DIALOG_MODAL /* flags */,
 				  GTK_MESSAGE_ERROR,
 				  GTK_BUTTONS_OK,
+				  FALSE /* markup */,
 				  _("Cannot start the greeter"),
 				  _("The greeter version (%s) does not match the daemon "
 				    "version.\n"
@@ -644,6 +648,7 @@ verify_gdm_version (void)
 				  GTK_DIALOG_MODAL /* flags */,
 				  GTK_MESSAGE_WARNING,
 				  GTK_BUTTONS_NONE,
+				  FALSE /* markup */,
 				  _("Cannot start the greeter"),
 				  _("The greeter version (%s) does not match the daemon "
 				    "version.\n"
@@ -688,6 +693,7 @@ verify_gdm_version (void)
 				  GTK_DIALOG_MODAL /* flags */,
 				  GTK_MESSAGE_WARNING,
 				  GTK_BUTTONS_NONE,
+				  FALSE /* markup */,
 				  _("Cannot start the greeter"),
 				  _("The greeter version (%s) does not match the daemon "
 				    "version (%s).\n"
@@ -740,6 +746,7 @@ greeter_message (const gchar *msg)
 			   GTK_DIALOG_MODAL /* flags */,
 			   GTK_MESSAGE_INFO,
 			   GTK_BUTTONS_OK,
+			   FALSE /* markup */,
 			   msg,
 			   /* avoid warning */ "%s", "");
   
@@ -754,7 +761,7 @@ greeter_message (const gchar *msg)
 
 
 gboolean
-greeter_query (const gchar *msg, const char *posbutton, const char *negbutton)
+greeter_query (const gchar *msg, gboolean markup, const char *posbutton, const char *negbutton)
 {
 	int ret;
 	GtkWidget *req;
@@ -767,6 +774,7 @@ greeter_query (const gchar *msg, const char *posbutton, const char *negbutton)
 				 GTK_DIALOG_MODAL /* flags */,
 				 GTK_MESSAGE_QUESTION,
 				 GTK_BUTTONS_NONE,
+				 markup,
 				 msg,
 				 /* avoid warning */ "%s", "");
 
@@ -956,8 +964,8 @@ greeter_reread_config (int sig, gpointer data)
 		g_free (GdmWelcome);
 		GdmWelcome = str;
 		if (welcome_string_info != NULL) {
-			g_free (welcome_string_info->orig_text);
-			welcome_string_info->orig_text = g_strdup (str);
+			g_free (welcome_string_info->data.text.orig_text);
+			welcome_string_info->data.text.orig_text = g_strdup (str);
 			greeter_item_update_text (welcome_string_info);
 		}
 	} else {
@@ -1128,10 +1136,10 @@ main (int argc, char *argv[])
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
 
-  greeter_parse_config ();
-
   setlocale (LC_ALL, "");
 
+  greeter_parse_config ();
+
   gtk_init (&argc, &argv);
 
   /* Should be a watch already, but anyway */
@@ -1164,23 +1172,28 @@ main (int argc, char *argv[])
   sigemptyset(&term.sa_mask);
   sigaddset (&term.sa_mask, SIGCHLD);
   
-  if (sigaction (SIGINT, &term, NULL) < 0) 
+  if G_UNLIKELY (sigaction (SIGINT, &term, NULL) < 0) 
     gdm_login_abort (_("%s: Error setting up %s signal handler: %s"), "main", "INT", strerror (errno));
   
-  if (sigaction (SIGTERM, &term, NULL) < 0) 
+  if G_UNLIKELY (sigaction (SIGTERM, &term, NULL) < 0) 
     gdm_login_abort (_("%s: Error setting up %s signal handler: %s"), "main", "TERM", strerror (errno));
   
-  sigfillset (&mask);
-  sigdelset (&mask, SIGTERM);
-  sigdelset (&mask, SIGHUP);
-  sigdelset (&mask, SIGINT);
-  sigdelset (&mask, SIGCHLD);
-  
-  if (sigprocmask (SIG_SETMASK, &mask, NULL) == -1) 
-    gdm_login_abort (_("Could not set signal mask!"));
+  sigemptyset (&mask);
+  sigaddset (&mask, SIGTERM);
+  sigaddset (&mask, SIGHUP);
+  sigaddset (&mask, SIGINT);
 
+  if G_UNLIKELY (sigprocmask (SIG_UNBLOCK, &mask, NULL) == -1) 
+	  gdm_login_abort (_("Could not set signal mask!"));
+
+  /* ignore SIGCHLD */
+  sigemptyset (&mask);
+  sigaddset (&mask, SIGCHLD);
+
+  if G_UNLIKELY (sigprocmask (SIG_BLOCK, &mask, NULL) == -1) 
+	  gdm_login_abort (_("Could not set signal mask!"));
   
-  if (! DOING_GDM_DEVELOPMENT) {
+  if G_LIKELY (! DOING_GDM_DEVELOPMENT) {
     ctrlch = g_io_channel_unix_new (STDIN_FILENO);
     g_io_channel_set_encoding (ctrlch, NULL, NULL);
     g_io_channel_set_buffered (ctrlch, FALSE);
@@ -1193,7 +1206,7 @@ main (int argc, char *argv[])
 
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 
-  if (DOING_GDM_DEVELOPMENT)
+  if G_UNLIKELY (DOING_GDM_DEVELOPMENT)
     g_signal_connect (G_OBJECT (window), "key_press_event",
 		      G_CALLBACK (key_press_event), NULL);
   
@@ -1230,23 +1243,26 @@ main (int argc, char *argv[])
 			gdm_wm_screen.height,
 			&error);
 
-    if (root == NULL &&
+    if G_UNLIKELY (root == NULL &&
 	g_getenv ("GDM_THEME") != NULL &&
 	DOING_GDM_DEVELOPMENT)
       {
         GtkWidget *dialog;
 	char *s;
+	char *tmp;
 
         gdm_wm_init (0);
         gdm_wm_focus_new_windows (TRUE);
     
+	tmp = ve_filename_to_utf8 (g_getenv ("GDM_THEME"));
 	s = g_strdup_printf (_("There was an error loading the "
-			       "theme %s"),
-			     g_getenv ("GDM_THEME"));
+			       "theme %s"), tmp);
+	g_free (tmp);
         dialog = ve_hig_dialog_new (NULL /* parent */,
 				    GTK_DIALOG_MODAL /* flags */,
 				    GTK_MESSAGE_ERROR,
 				    GTK_BUTTONS_OK,
+				    FALSE /* markup */,
 				    s,
 				    "%s", (error && error->message) ? error->message : "");
 	g_free (s);
@@ -1261,11 +1277,11 @@ main (int argc, char *argv[])
         exit(1);
       }
 
-  if (error)
+  if G_UNLIKELY (error)
     g_clear_error (&error);
 
   /* Try circles.xml */
-  if (root == NULL)
+  if G_UNLIKELY (root == NULL)
     {
       g_free (theme_file);
       g_free (theme_dir);
@@ -1277,7 +1293,7 @@ main (int argc, char *argv[])
 			    NULL);
     }
 
-  if (root != NULL && greeter_lookup_id ("user-pw-entry") == NULL)
+  if G_UNLIKELY (root != NULL && greeter_lookup_id ("user-pw-entry") == NULL)
     {
       GtkWidget *dialog;
 
@@ -1288,6 +1304,7 @@ main (int argc, char *argv[])
 				  GTK_DIALOG_MODAL /* flags */,
 				  GTK_MESSAGE_ERROR,
 				  GTK_BUTTONS_OK,
+				  FALSE /* markup */,
 				  _("The theme for the graphical greeter "
 				    "is corrupt"),
 				  "%s",
@@ -1306,13 +1323,13 @@ main (int argc, char *argv[])
     }
 
   /* FIXME: beter information should be printed */
-  if (DOING_GDM_DEVELOPMENT && root == NULL)
+  if G_UNLIKELY (DOING_GDM_DEVELOPMENT && root == NULL)
     {
       g_warning ("No theme could be loaded");
       exit(1);
     }
 
-  if (root == NULL)
+  if G_UNLIKELY (root == NULL)
     {
       GtkWidget *dialog;
 
@@ -1323,6 +1340,7 @@ main (int argc, char *argv[])
 				  GTK_DIALOG_MODAL /* flags */,
 				  GTK_MESSAGE_ERROR,
 				  GTK_BUTTONS_OK,
+				  FALSE /* markup */,
 				  _("There was an error loading the "
 				    "theme, and the default theme "
 				    "also could not have been loaded, "
@@ -1344,6 +1362,7 @@ main (int argc, char *argv[])
 				  GTK_DIALOG_MODAL /* flags */,
 				  GTK_MESSAGE_ERROR,
 				  GTK_BUTTONS_OK,
+				  FALSE /* markup */,
 				  _("I could not start the standard "
 				    "greeter.  This display will abort "
 				    "and you may have to login another "
@@ -1372,7 +1391,7 @@ main (int argc, char *argv[])
   setup_background_color ();
 
   /* can it ever happen that it'd be NULL here ??? */
-  if (window->window != NULL)
+  if G_UNLIKELY (window->window != NULL)
     {
       gdm_wm_init (GDK_WINDOW_XWINDOW (window->window));
 
@@ -1382,7 +1401,7 @@ main (int argc, char *argv[])
       gdm_wm_focus_window (GDK_WINDOW_XWINDOW (window->window));
     }
 
-  if (session_dir_whacked_out)
+  if G_UNLIKELY (session_dir_whacked_out)
     {
       GtkWidget *dialog;
 
@@ -1392,6 +1411,7 @@ main (int argc, char *argv[])
 				  GTK_DIALOG_MODAL /* flags */,
 				  GTK_MESSAGE_ERROR,
 				  GTK_BUTTONS_OK,
+				  FALSE /* markup */,
 				  _("Session directory is missing"),
 				  "%s",
 				  _("Your session directory is missing or empty!  "
@@ -1408,7 +1428,7 @@ main (int argc, char *argv[])
       gdm_wm_no_login_focus_pop ();
     }
 
-  if (g_getenv ("GDM_WHACKED_GREETER_CONFIG") != NULL)
+  if G_UNLIKELY (g_getenv ("GDM_WHACKED_GREETER_CONFIG") != NULL)
     {
       GtkWidget *dialog;
 
@@ -1418,6 +1438,7 @@ main (int argc, char *argv[])
 				  GTK_DIALOG_MODAL /* flags */,
 				  GTK_MESSAGE_ERROR,
 				  GTK_BUTTONS_OK,
+				  FALSE /* markup */,
 				  _("Configuration is not correct"),
 				  "%s",
 				  _("The configuration file contains an invalid command "
@@ -1436,7 +1457,7 @@ main (int argc, char *argv[])
   }
 
   /* There was no config file */
-  if (used_defaults)
+  if G_UNLIKELY (used_defaults)
     {
       GtkWidget *dialog;
 
@@ -1446,6 +1467,7 @@ main (int argc, char *argv[])
 				  GTK_DIALOG_MODAL /* flags */,
 				  GTK_MESSAGE_ERROR,
 				  GTK_BUTTONS_OK,
+				  FALSE /* markup */,
 				  _("No configuration was found"),
 				  "%s",
 				  _("The configuration was not found.  GDM is using "
