@@ -1,7 +1,7 @@
 /* Simple pam thingie to ask the root password, for use in XKeepsCrashing
  * script. */
 #include "config.h"
-#include <gnome.h>
+#include <libgnome/libgnome.h>
 #include <stdio.h>
 #include <locale.h>
 #include <unistd.h>
@@ -32,8 +32,8 @@ main (int argc, char *argv[])
 	}
 
 	setlocale (LC_ALL, "");
-	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
-	textdomain (PACKAGE);
+	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
+	textdomain (GETTEXT_PACKAGE);
 
 	for (tries = 3; tries > 0; tries --) {
 		if ((retval = pam_start ("gdm", username, &conv, &pamh)) != PAM_SUCCESS) {
